# -*- coding: utf-8 -*-
"""
Created on Sat Oct 26 19:46:10 2019

@author: Yanan Jia
"""
import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import fsolve
'''
===============================================================================
*************Figures for a PR regulated farmer's optimal strategies************
  Based on optimal strategies solved by the model, we draw figures in several 
steps:
  (1) Under any cost parameters (b,d,v), we can figure out a PR regulated farmer's 
  optimal strategy according to the model's prediction. We categorize every 
  points in (b,d,v) spcace by their corresponding optimal strategies. 
  We illustrate optimal strategies in b-v plane by holding self-test cost fixed.
  The b-v plane is devided into several areas which represent different optimal
  strategies.
  In order to observe how the representation of optimal strategies in b-v
  plane changes as self-test cost changes, we draw figures by holding
  self-test cost at different levels.
    (1-1) Define a rule which we use to categrize points (b,d,v).These are exact 
    conditions we drived from optimization problem.
===============================================================================
'''
# function color() can take several values: "Te1", "NTC1", "C1", "C2".
def color(b,v):
    if ((v<=l3-l2 and b<=l2-l1) \
       or (v>l3-l2 and  b<=l2-l1 and b<=l3-l1-v and d>(1-beta)*(l2+v-l3) and \
           b<=(l3-(1-beta)*l2-v)/beta-l1) \
       or (v>l3-l2 and  b<=l2-l1 and b>l3-l1-v and b<=(l3-(1-beta)*l2-v)/beta-l1)):
        return "C1"
    elif (v<=l3-l2 and b>l2-l1):
        return "C2"
    elif (v>l3-l2 and b<=l2-l1 and b<=l3-l1-v and d<=(1-beta)*(l2+v-l3) and \
          d<=beta*(l3-l1-b-v)):
        return "Te1"
    elif ((v>l3-l2 and  b<=l2-l1 and b<=l3-l1-v and d>beta*(l3-l1-b-v) and \
           b>(l3-(1-beta)*l2-v)/beta-l1) \
       or (v>l3-l2 and  b<=l2-l1 and b>l3-l1-v and b>(l3-(1-beta)*l2-v)/beta-l1) \
       or (v>l3-l2 and  b>l2-l1)):
        return "NTC1"
# Label values of function color(b,v)
# Function color (b,v) never takes value of Te2, Te3 and NTC2. We prepare Te2, 
# Te3 and NTC2 for figures for an unregulated farmer's optimal strategies. 
#C1=Call; for 'E' infection treat, not treat if 'I'
#C2=Call; never treat
#NTC1=Neither call nor test; never treat 
#NTC2=Neither call nor test; always treat
#Te1=Test; for 'E' infection call and treat; for 'I' infection neither call nor treat
#Te2=Test; for 'E' infection do not call but treat; for 'I' infection call and do not treat
#Te3=Test; never call; for 'E' infection treat; for 'I' infection do not treat
'''
    (1-2) In theory, we consider every points in (b,d,v) space. In practice, 
    we consider points that evenly spread within the b-v plane. As long as the 
    distance between every two points are close enough, we can tell when
    the optimal strategy changes. 

    We consider the points in (0,600) range on b-axis and in (0,600) range on v-axis. 
    We choose these ranges so that we draw a comprehensive representation of optimal 
    strategy in b-v plane. 
    The distances between two points in b-axis and v-axis directions are assumed to 
    be 0.5 and 1 respectively. we picked these values since the distance is close 
    enough to present main findings. We can choose smaller distance, then it cost 
    more time to fill figures with colors.
'''        
# In order to plot the figure, we need to set values for other parameters. We 
# draw figures by holding self-test cost fixed at 10 and 60. 
        
beta,l1,l2,l3,d=0.5,50,250,600,10  #d=10,60
b = np.arange(0,600,0.5) 
v = np.arange(0,600,1)
'''
    (1-3) The following step categorize points by Function color(b,v) into 
    several areas which represent different optimal strategies.
'''        
area = [[x,y] for x in b for y in v] 
area_Te1 = [a for a in area if color(a[0],a[1]) == "Te1"]
area_C1 = [a for a in area if color(a[0],a[1]) == "C1"] 
area_C2 = [a for a in area if color(a[0],a[1])=="C2"]
area_NTC1 = [a for a in area if color(a[0],a[1]) == "NTC1"] 

'''
    (1-4) When we draw b-v plane by holding parameter d fixed, we can colored
        points according to their categories.
'''   
# set the size of the figure
fig=plt.figure(figsize=(15,7))
ax1=fig.add_subplot(111) 

# colored each area
# Area with optimal strategy Te1 is colored orange
# Area with optimal strategy C1 is colored lightcoral
# Area with optimal strategy C2 is colored lightgray
# Area with optimal strategy NTC1 is colored brown
area = [[x,y] for x in b for y in v] 
ax1.scatter([a[0] for a in area_Te1],[a[1] for a in area_Te1],c='orange', \
            linewidths=3,marker='.',alpha = 0.05)
ax1.scatter([a[0] for a in area_C1],[a[1] for a in area_C1],c='lightcoral', \
            linewidths=3,marker='.', alpha = 0.05)
ax1.scatter([a[0] for a in area_C2],[a[1] for a in area_C2],c='lightgray', \
            linewidths=3,marker='.',alpha = 0.05)
ax1.scatter([a[0] for a in area_NTC1],[a[1] for a in area_NTC1],c='tab:brown', \
            linewidths=3,marker='.',alpha = 0.05)
'''       
    (2) In stead of using colored figures to illustrate how optimal strategies 
        varies across cost parameters, we prefer line artworks for paper submission. 
        Therefore we draw lines representing indifferent conditions across which 
        the farmer's optimal strategy switch. In line artworks, the b-v plane are
        separated into several areas by lines. Across the areas, optimal strategy
        may differ. Situations exist where two adjacent areas are corresponding 
        to the same optimal strategy. For these adjacent areas, we remove the 
        indifferent condition lines between them to keep the figure clean without 
        a loss of information.    
'''
# The indifferent conditions are solved from the maximization problem. We reorganize 
# these indifferent conditions at different levels of self-test cost.

# In some lines, we add remark "#" at the beginning so that the code is inactive.
# As we explained above, we deactive these code and so remove the corresponding 
# lines to make the figures clean but do not loss information. 
         
v1=l3-l2+0*b
v2=l3-beta*(l1+b)-(1-beta)*l2
v3=l3-l1-b
v4=d/(1-beta)-l2+l3+0*b
v5=l3-l1-b-d/beta
b1=(l2-l1)+0*v
in1=fsolve(lambda b: l3-beta*(l1+b)-(1-beta)*l2-(l3-l1-b-d/beta),0) 
#intersection between v2 and v5

if d<beta*(1-beta)*(l2-l1):
    ax1.plot(b[np.where(b>b1[0])],v1[np.where(b>b1[0])], 'k--', \
             label=r'$v=l_3-l_2$')
    ax1.plot(b[np.where((b<b1[0]) & (b>in1[0]))],v2[np.where((b<b1[0]) & (b>in1[0]))], \
             'k-', label=r'$v=l_3-\beta*(l_1+b)-(1-\beta)*l_2$')
        #ax1.plot(b,v3, 'k-', label=r'$v=l_3-l_1-b$')
    ax1.plot(b[np.where(b<in1[0])],v4[np.where(b<in1[0])], 'k--', \
             label=r'$v=d/(1-\beta)-l_2+l_3$')
    ax1.plot(b[np.where(b<in1[0])],v5[np.where(b<in1[0])], 'k--', \
             label=r'$v=l_3-l_1-b-d/\beta$')
    ax1.plot(b1[np.where(v<v1[0])],v[np.where(v<v1[0])], 'k--', \
             label=r'$b=l_2-l_1$')
elif d>beta*(1-beta)*(l2-l1):
    ax1.plot(b[np.where(b>b1[0])],v1[np.where(b>b1[0])], 'k--', \
             label=r'$v=l_3-l_2$')
    ax1.plot(b[np.where(b<b1[0])],v2[np.where(b<b1[0])], 'k--', \
             label=r'$v=l_3-\beta*(l_1+b)-(1-\beta)*l_2$')
        #ax1.plot(b,v3, 'k--', label=r'$v=l_3-l_1-b$')
        #ax1.plot(b,v4, 'k--', label=r'$v=d/(1-\beta)-l_2+l_3$')
        #ax1.plot(b,v5, 'k--', label=r'$v=l_3-l_1-b-d/\beta$')
    ax1.plot(b1[np.where(v<v1[0])],v[np.where(v<v1[0])], 'k--', label=r'$b=l_2-l_1$')
'''       
    (3) According to colors, we can learn the corresponding optimal strategies 
    in each area. Then we deactive the code of coloring and so remove colors
    for submission. To deactive the code, we just need to add remark "#"
    at the beginning of line 96 through 99.                 
'''

'''       
    (4) General setting for figures: the axis labels, and where to 
    save these figures.                 
'''
# Horizontal axis is labeled as Antibiotic cost. Vertical axis is labeled as 
# Veterinary service cost.
plt.xlabel('Antibiotic cost b', fontsize='25')
plt.ylabel('Veterinary service cost v', fontsize='25')
# set axis labels and ranges.     
ax1.set_xticklabels([])
ax1.set_yticklabels([])
ax1.set_xlim(0,600)
ax1.set_ylim(0,600)
# Name the figure depending on cost parameters
cat="NA"
if d<beta*(1-beta)*(l2-l1):
    cat="PR_Lowd_"
elif d>beta*(1-beta)*(l2-l1):
    cat="PR_Highd_"
path = 'C:\\Users\\Yanan Jia\\Dropbox\\RA\\First dissertation paper\\Figure\\b-v\\'
plt.savefig(path+cat+'b-v_PR.png',dpi = 800,bbox_inches = 'tight') 
plt.show()
'''
Note: We provide line artworks instead of colored figures in submitted paper.
'''